# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.


"""
Fabric tasks for bootstrapping, installing, deploying and running OpenAIRE.

Examples
--------
Bootstrap OpenAIRE with Python 2.4 and copy production data
  fab loc:py=24 bootstrap openaire_fixture_load

Configure OpenAIRE repository to use with environment
  fab loc:py=24,ref=master repo_configure:openaire
"""

from fabric.api import task, abort, puts, local, roles, sudo, cd, execute
from fabric.colors import cyan
from fabric.contrib.console import confirm
from inveniofab.api import *
from inveniofab.utils import template_hook_factory
from jinja2.exceptions import TemplateNotFound
import os


@task
def int(activate=True, **kwargs):
    """ Environment: Integration """
    env = env_create('int', activate=activate, **kwargs)

    env.roledefs = {
        'web': ['devhost', ],
        'lb': [],
        'db-master': ['devhost', ],
        'db-slave': [],
        'workers': ['devhost', ],
    }

    env.CFG_SRCDIR = '/opt/src'
    env.CFG_INVENIO_SRCDIR = os.path.join(env.CFG_SRCDIR, 'invenio')
    env.CFG_INVENIO_PREFIX = '/opt/invenio'
    env.CFG_INVENIO_CONF = 'etc/invenio-local.conf'
    env.CFG_INVENIO_HOSTNAME = "devhost"
    env.CFG_INVENIO_DOMAINNAME = ""
    env.CFG_INVENIO_PORT_HTTP = "80"
    env.CFG_INVENIO_PORT_HTTPS = "443"
    env.CFG_INVENIO_USER = 'apache'
    env.CFG_INVENIO_APACHECTL = '/etc/init.d/httpd'
    env.CFG_INVENIO_ADMIN = 'openaire-admin@cern.ch'

    env.CFG_OPENAIRE_SRCDIR = os.path.join(env.CFG_SRCDIR, 'openaire')
    env.CFG_OPENAIRE_PORTAL_URL = "https://test.openaire.eu"

    env.CFG_INVENIO_REPOS = [
        ('invenio', {
            'repository': 'http://invenio-software.org/repo/invenio/',
            'ref': 'origin/maint-1.1',
            'bootstrap_targets': ['all', 'install', 'install-mathjax-plugin', 'install-ckeditor-plugin', 'install-pdfa-helper-files', 'install-jquery-plugins', ],
            'deploy_targets': ['all', 'install', ],
        }),
        ('openaire', {
            'repository': 'http://invenio-software.org/repo/openaire/',
            'ref': 'origin/maint',
            'bootstrap_targets': ['all', 'install', ],
            'deploy_targets': ['all', 'install', ],
        }),
    ]

    env.CFG_DATABASE_DUMPDIR = env.CFG_INVENIO_PREFIX
    env.CFG_DATABASE_HOST = 'somedbhost'
    env.CFG_DATABASE_PORT = 3306
    env.CFG_DATABASE_NAME = 'somedb'
    env.CFG_DATABASE_USER = 'someuser'
    env.CFG_DATABASE_PASS = 'somepassword'
    env.CFG_DATABASE_DROP_ALLOWED = True


@task
def loc(activate=True, py=None, ref=None, invenio='master', **kwargs):
    """
    Local environment (example: loc:py=24,ref=master)
    """
    env = env_create('loc', name=env_make_name('openaire', py or '', ref or ''),
                     activate=activate, python=py, **kwargs)

    # Do not use git-new-workdir
    env.WITH_WORKDIR = False
    env.CFG_SRCWORKDIR = env.CFG_SRCDIR
    
    # Setup Invenio/OpenAIRE source directories (works just as short-cuts)
    # Their path must match CFG_SRCDIR/<reponame> where <reponame> is defined 
    # in CFG_INVENIO_REPOS.
    env.CFG_INVENIO_SRCDIR = os.path.join(env.CFG_SRCDIR, 'invenio')
    env.CFG_OPENAIRE_SRCDIR = os.path.join(env.CFG_SRCDIR, 'openaire')

    # OPENAIRE related variable, needed in invenio-local.conf
    env.CFG_OPENAIRE_PORTAL_URL = ""

    # Append OpenAIRE repository
    env.CFG_INVENIO_REPOS += [
        ('openaire', {
            'repository': 'http://invenio-software.org/repo/openaire/',
            'ref': ref or 'maint',
            'bootstrap_targets': ['all', 'install', ],
            'deploy_targets': ['all', 'install', ],
            'requirements' : ['%(CFG_OPENAIRE_SRCDIR)s/requirements.txt'],
            #'configure_hook': template_hook_factory('config-local.mk','%(topsrcdir)s/config-local.mk'),
            #'prepare_hook': None, # Don't run any hook on prepare
        }),
    ]

    # Set default Invenio branch
    env = env_override(env, 'invenio', invenio, override={})
    return env

# =======================
# OpenAIRE specific tasks
# =======================
@task
def openaire_createdb():
    """ Create OpenAIRE tables """
    from fabric.api import env
    puts(cyan(">>> Creating OpenAIRE tables..." % env))
    ctx = {
        'topsrcdir': repo_check('openaire', check_path=True, workdir=True),
    }
    ctx.update(env)
    local("cd %(topsrcdir)s; make install-tables" % ctx)


@task
def openaire_fixture_load(quite=False):
    """ Fixture: Copy metadata and data from production """
    from fabric.api import env
    puts(cyan(">>> Loading OpenAIRE fixtures..." % env))

    def _confirm_step(func, *args, **kwargs):
        if quite or confirm(cyan("Run step %s?" % func.__name__)):
            func(*args, **kwargs)

    if quite or confirm(cyan("Run step copy_dump")):
        puts(cyan(">>> Getting latest database dump..." % env))
        local("cp `ls -1t /afs/cern.ch/project/cds/OpenAIRE/*.gz | head -n 1` %(CFG_INVENIO_PREFIX)s/%(CFG_DATABASE_NAME)s.sql.gz" % env)

    _confirm_step(mysql_load)
    _confirm_step(test_clean)

    if quite or confirm(cyan("Run step database_upgrade")):
        puts(cyan(">>> Running database upgrade..." % env))
        local("%(CFG_INVENIO_PREFIX)s/bin/inveniocfg --upgrade --yes-i-know" %
              env)

    # Specific to OpenAIRE
    if quite or confirm(cyan("Run step reset_admin")):
        local("""echo "UPDATE user SET password=AES_ENCRYPT(email,''), nickname='admin' WHERE nickname='lnielsen';" | %(CFG_INVENIO_PREFIX)s/bin/dbexec""" % env)
        test_reset_admin()

    ctx = {
        'files': " ".join([
            "etc/bibclassify/eurovoc-en.txt",
            "etc/bibconvert/config/OpenAIRE.tpl",
            "etc/bibconvert/config/OpenAIREmodify.tpl",
            "etc/bibexport/OpenAIRE-sitemap.cfg",
            "etc/bibrank/combine_method.cfg",
            "etc/bibrank/download_similarity.cfg",
            "etc/bibrank/download_total.cfg",
            "etc/bibrank/download_users.cfg",
            "etc/webaccess/robot_keys.dat",
        ]),
    }
    ctx.update(env)

    if quite or confirm(cyan("Run step copy_files")):
        puts(cyan(">>> Copying data files and fixing record URLs..." % env))
        local("""cd %(CFG_INVENIO_PREFIX)s;ssh openaire.cern.ch "cd /opt/invenio && tar -zvcf - %(files)s" | tar xzf -""" % ctx)
        local("""rsync -e ssh --delete -rLptgoDv root@openaire.cern.ch:/opt/invenio/var/data/ %(CFG_INVENIO_PREFIX)s/var/data/""" % env)

    _confirm_step(openaire_fixture_fix_links)
    _confirm_step(openaire_bibsched_tasks)
    puts(cyan(">>> OpenAIRE fixtures loaded..." % env))


@task
def openaire_fixture_fix_links():
    """ Fixture: Change external to internal links """
    from fabric.api import env

    local("rm -Rf %(CFG_INVENIO_PREFIX)s/fix_8564_output.xml" % env)
    local("""%(CFG_INVENIO_PREFIX)s/bin/python %(CFG_OPENAIRE_SRCDIR)s/miscutil/lib/fix_8564.py > %(CFG_INVENIO_PREFIX)s/fix_8564_output.xml""" % env)
    local("""echo "TRUNCATE schTASK;" | %(CFG_INVENIO_PREFIX)s/bin/dbexec""" %
          env)
    local("""%(CFG_INVENIO_PREFIX)s/bin/bibupload -c %(CFG_INVENIO_PREFIX)s/fix_8564_output.xml""" % env)
    local("""%(CFG_INVENIO_PREFIX)s/bin/bibupload 1""" % env)
    local("rm -Rf %(CFG_INVENIO_PREFIX)s/fix_8564_output.xml" % env)


@task
def openaire_test(quite=False):
    """
    Test OpenAIRE
    """
    from fabric.api import env
    puts(cyan(">>> Preparing OpenAIRE test environment..." % env))

    # Load archived environment and re-install OpenAIRE
    test_load(quite=quite)

    if quite or confirm(cyan("Run step test?")):
        puts(cyan(">>> Running tests..." % env))
        local('%(CFG_INVENIO_PREFIX)s/bin/python %(CFG_OPENAIRE_SRCDIR)s/openaire/lib/openaire_deposit_engine_tests.py' % env)
        local('%(CFG_INVENIO_PREFIX)s/bin/python %(CFG_OPENAIRE_SRCDIR)s/openaire/lib/openaire_deposit_webinterface_tests.py' % env)

    if quite or confirm(cyan("Run step openaire_bibsched_tasks?")):
        openaire_bibsched_tasks()


@task
def openaire_bibsched_tasks():
    from fabric.api import env
    puts(cyan(
        ">>> Running bibindex, bibrank, bibreformat and webcoll..." % env))
    local('echo "TRUNCATE schTASK;" | %(CFG_INVENIO_PREFIX)s/bin/dbexec' % env)
    local('%(CFG_INVENIO_PREFIX)s/bin/bibindex -f50000 -s5m -u admin' % env)
    local('%(CFG_INVENIO_PREFIX)s/bin/bibindex 1' % env)
    local('%(CFG_INVENIO_PREFIX)s/bin/bibrank -f50000 -s5m -u admin' % env)
    local('%(CFG_INVENIO_PREFIX)s/bin/bibrank 2' % env)
    local('%(CFG_INVENIO_PREFIX)s/bin/bibreformat -s5m -u admin' % env)
    local('%(CFG_INVENIO_PREFIX)s/bin/bibreformat 3' % env)
    local('%(CFG_INVENIO_PREFIX)s/bin/webcoll -s5m -u admin' % env)
    local('%(CFG_INVENIO_PREFIX)s/bin/webcoll 4' % env)
    local('%(CFG_INVENIO_PREFIX)s/bin/bibtasklet -T bst_openaire_keywords -u admin -s1d' % env)
    local('%(CFG_INVENIO_PREFIX)s/bin/bibtasklet 5' % env)
    local(
        '%(CFG_INVENIO_PREFIX)s/bin/oairepositoryupdater -s1h -u admin' % env)
    local('%(CFG_INVENIO_PREFIX)s/bin/oairepositoryupdater 6' % env)

# ==========================
# OpenAIRE remote deployment
# ==========================


@roles('web')
@task
def deploy(**repos):
    """ Deployment: Prepare source """
    from fabric.api import env

    for repo, ref  in repos.items():
        repo_check(repo, check_path=False)

    for repo, ref  in repos.items():
        puts(" - %s at %s" % (repo, ref))

    if not confirm(cyan("Deploy above repository?")):
        abort("Deployment aborted.")

    sudo("%(CFG_INVENIO_PREFIX)s/bin/bibsched stop" % env, user=env.CFG_INVENIO_USER)
    execute(deploy_source, **repos)
    execute(deploy_install, **repos)
    sudo("%(CFG_INVENIO_PREFIX)s/bin/bibsched start" % env, user=env.CFG_INVENIO_USER)
    execute(apache_restart)


@roles('web')
@task
def deploy_source(**repos):
    """ Deployment: Prepare source """
    from fabric.api import env

    # Check all repositories first
    for repo, ref  in repos.items():
        repo_check(repo, check_path=False)

    for repo, dummy_info  in env.CFG_INVENIO_REPOS:
        if repo in repos:
            ref = repos[repo]
            with cd(os.path.join(env.CFG_SRCDIR, repo)):
                sudo("git fetch", user=env.CFG_INVENIO_USER)
                sudo("git reset --hard %s" % ref, user=env.CFG_INVENIO_USER)
                sudo("make -s clean", user=env.CFG_INVENIO_USER)
                sudo("make -s", user=env.CFG_INVENIO_USER)


@roles('web')
@task
def deploy_install(**repos):
    """ Deployment: Install source code """
    from fabric.api import env

    for repo in repos.keys():
        repo_check(repo, check_path=False)

    for repo, dummy_info  in env.CFG_INVENIO_REPOS:
        if repo in repos:
            with cd(os.path.join(env.CFG_SRCDIR, repo)):
                sudo("make -s install", user=env.CFG_INVENIO_USER)

    sudo("%(CFG_INVENIO_PREFIX)s/bin/inveniocfg --update-all" % env, user=env.CFG_INVENIO_USER)
    sudo("%(CFG_INVENIO_PREFIX)s/bin/inveniocfg --upgrade" % env, user=env.CFG_INVENIO_USER)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

# FIXME: Move to fabfile.py for INSPIRE?

from fabric.api import task, puts, local, env
from fabric.colors import cyan
from inveniofab.git import repo_check

@task
def inspire_dbchanges():
    """ Perform INSPIRE db changes """
    puts(cyan(">>> Updating tables from INSPIRE..." % env))

    ctx = {
        'topsrcdir': repo_check('inspire', check_path=True)
    }
    ctx.update(env)

    local("cd %(topsrcdir)s; make install-dbchanges" % ctx)

    # Look for the webcoll task id added by install-dbchanges and run it
    query_output = local('echo "select max(id) from schTASK;" | %(CFG_INVENIO_PREFIX)s/bin/dbexec' % ctx,
                        capture=True)
    task_id = query_output.split('\n')[-1]

    puts(cyan(">>> Running webcoll task %s..." % task_id))
    ctx.update({'webcoll_task_id': task_id})
    local("%(CFG_INVENIO_PREFIX)s/bin/webcoll %(webcoll_task_id)s" %
          ctx)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Compound tasks to perform bigger operations like bootstrapping Invenio.
"""

from fabric.api import task, env
from fabric.colors import cyan
from fabric.contrib.console import confirm
from inveniofab.devserver import devserver_conf
from inveniofab.git import repo_update, repo_install
from inveniofab.invenio import invenio_conf, invenio_createdb, \
    invenio_upgrade
from inveniofab.mysql import mysql_createdb, mysql_dump, mysql_load, \
    mysql_dropdb
from inveniofab.venv import venv_create, venv_requirements, venv_dump, \
    venv_load, venv_drop
# FIXME: See note in bootstrap() method regarding inspire_dbchanges
from inveniofab.inspire import inspire_dbchanges


@task
def bootstrap(with_db=True, quite=False, **kwargs):
    """
    Bootstrap an Invenio installation
    
    Bootstrap will run the following tasks:
    
     * ``mysql_dropdb`` - to drop an existing database if it exists.
     * ``mysql_createdb`` - to create database and user.
     * ``venv_create`` - to create a virtual environment.
     * ``repo_update`` - to checkout source code from repositories.
     * ``venv_requirements`` - to install Python requirements.
     * ``repo_install`` - to install all repositories.
     * ``invenio_conf`` - to configure Invenio.
     * ``devserver_conf`` - to configure the development server.
     * ``invenio_createdb`` - to create Invenio database tables.

    If a task fails, you can re-run bootstrap and skip the initial steps which
    already was completed.

    :param quite: Default ``False``. Set to ``True`` to disable user confirmation.
    :param with_db: Default ``True``. Run database related tasks to create the database.
    """

    def _confirm_step(func, *args, **kwargs):
        if quite or confirm(cyan("Run step %s?" % func.__name__)):
            func(*args, **kwargs)

    if with_db:
        _confirm_step(mysql_dropdb)
        _confirm_step(mysql_createdb)
    _confirm_step(venv_create)
    _confirm_step(repo_update, **kwargs)
    _confirm_step(venv_requirements)
    _confirm_step(repo_install, targets_key='bootstrap_targets')
    _confirm_step(invenio_conf)
    if env.WITH_DEVSERVER:
        _confirm_step(devserver_conf)
    if with_db:
        _confirm_step(invenio_createdb)
    # FIXME: Generalize below to have hooks in the bootstrap process, which can be
    # customized for each individual overlay (e.g. OpenAIRE/INSPIRE needs to create
    # extra tables etc., load demo records?)
    if env.CFG_INSPIRE_SITE:
        _confirm_step(inspire_dbchanges)


@task
def install(quite=False, **kwargs):
    """
    Install repositories
    
    The task will checkout latest changes for the repositories, run make install,
    ``inveniocfg --update-all --upgrade``.
    
    :param quite: Default ``False``. Set to ``True`` to disable user confirmation.
    """

    def _confirm_step(func, *args, **kwargs):
        if quite or confirm("Run step %s?" % func.__name__):
            func(*args, **kwargs)

    _confirm_step(repo_update, **kwargs)
    _confirm_step(repo_install, targets_key='deploy_targets')
    _confirm_step(invenio_conf)
    _confirm_step(invenio_upgrade)


@task
def dump():
    """
    Archive installation
    
    Dump a database and virtual environment to an archive which can later be
    restored with :meth:`~load`.
    """
    mysql_dump()
    venv_dump()


@task
def load():
    """
    Load archived installation
    
    Load an archived virtual environment and database which was dumped with
    :meth:`~dump`.
    """
    venv_load()
    mysql_load()


@task
def drop():
    """
    Remove installation
    
    Remove virtual environment, database and database user.
    """
    venv_drop()
    mysql_dropdb()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Tasks for creating and installing Python virtual environments.

All task works on local system.
"""

from fabric.api import task, puts, env, local, abort
from fabric.contrib.console import confirm
from fabric.colors import red, cyan
from inveniofab.utils import write_template, python_version
import os


@task
def venv_pyuno_install():
    """
    Install Python OpenOffice binding

    The tassk will try to locate ``uno.py`` and ``unohelper.py`` in ``/usr/``,
    and copy it to your virtualenv's site-packages.

    .. warning::
      
       The Python OpenOffice bindings from your system is specific to
       your system's Python interpreter, hence if your system Python is 2.7 and 
       you are installing the bindings into virtualenv with Python 2.4, the 
       bindings will not work.
    """
    pyver = python_version()
    ctx = {'pyver': pyver}
    ctx.update(env)

    local("cp  `find /usr/ -name uno.py 2>/dev/null | head -n 1` %(CFG_INVENIO_PREFIX)s/lib/python%(pyver)s/site-packages/" % ctx)
    local("cp `find /usr/ -name unohelper.py 2>/dev/null | head -n 1` %(CFG_INVENIO_PREFIX)s/lib/python%(pyver)s/site-packages/" % ctx)


@task
def venv_requirements():
    """
    Install Python packages
    
    The task will install Python packages defined in PIP requirements file.
    """
    requirements = []
    for repo, info in env.CFG_INVENIO_REPOS:
        requirements.extend([(repo, x) for x in info.get('requirements', [])])

    if not requirements:
        puts(cyan(">>> No requirements defined..."))
        return

    puts(cyan(">>> Installing requirements..." % env))

    pyver = python_version()

    for repo, reqfile in requirements:
        reqfile = reqfile % env
        reqpath = os.path.join(env.CFG_INVENIO_PREFIX, "%s_%s" % (repo, os.path.basename(reqfile)))

        ctx = {'reqpath': reqpath, 'reqfile': reqfile, 'pyver': pyver}
        ctx.update(env)

        puts(">>> Writing requirements to %(reqpath)s ..." % ctx)
        write_template(reqpath, ctx, tpl_file=reqfile)

        puts(">>> Installing requirements from %(reqpath)s ..." % ctx)
        cmds = [env.ACTIVATE] if env.WITH_VIRTUALENV else []
        cmds.append("pip install -r %(reqpath)s")
        local(" && ".join(cmds) % ctx)


@task
def venv_dump():
    """
    Archive a virtualenv
    
    The task will create an archive ``<virtualenv name>.tar.gz`` of the entire
    virtual environment. If an existing archive already exists, the user will
    be asked for confirmation to remove it. Normally this command is invoked
    indirectly via the compound task :meth:`inveniofab.compound.dump` which 
    takes care of dumping the database prior to archiving the virtual 
    environment.
    """
    puts(cyan(">>> Creating archive of virtualenv in %(CFG_INVENIO_PREFIX)s..." % env))

    ctx = {
        'dirname' : os.path.dirname(env.CFG_INVENIO_PREFIX),
        'basename' : os.path.basename(env.CFG_INVENIO_PREFIX),
    }

    archive_file = "%(dirname)s/%(basename)s.tar.gz" % ctx
    if os.path.exists(archive_file):
        res = confirm("Existing archive already exists - remove?")
        if not res:
            abort(red("Cannot continue") % env)
        else:
            local("rm -Rf %s" % archive_file)

    cmds = [
        "cd %(dirname)s",
        "tar -cvzf %(basename)s.tar.gz %(basename)s",
    ]
    local(" && ".join(cmds) % ctx)


@task
def venv_load():
    """
    Load an archived virtualenv
    
    The task will extract an archived virtual environment created with 
    :meth:`~venv_dump`. Normally this command is invoked
    indirectly via the compound task :meth:`inveniofab.compound.load` which 
    takes care of loading the database after extracting the virtual 
    environment.
    """
    puts(cyan(">>> Loading archived virtualenv..."))

    ctx = {
        'dirname' : os.path.dirname(env.CFG_INVENIO_PREFIX),
        'basename' : os.path.basename(env.CFG_INVENIO_PREFIX),
    }

    archive_file = "%(dirname)s/%(basename)s.tar.gz" % ctx
    if not os.path.exists(archive_file):
        abort(red("Archived virtualenv does not exists - cannot continue") % env)

    # Remove previous installation
    if os.path.exists(env.CFG_INVENIO_PREFIX):
        res = confirm("Remove installation in %(CFG_INVENIO_PREFIX)s ?" % env)
        if not res:
            abort(red("Cannot continue") % env)
        else:
            local("sudo rm -Rf %(CFG_INVENIO_PREFIX)s" % env)

    cmds = [
        "cd %(dirname)s",
        "tar -xvzf %(basename)s.tar.gz",
    ]
    local(" && ".join(cmds) % ctx)


@task
def venv_create():
    """
    Create virtualenv environment
    
    The virtualenv is created in ``env.CFG_INVENIO_PREFIX``, and will also
    create ``lib/python/invenio/`` and symlink it the virtualenv's 
    site-packages, as well as ``var/tmp/ooffice-tmp-files`` (via sudo). If 
    ``env.WITH_DEVSCRIPTS`` is ``True``, invenio-devscripts will be installed. 
    If ``env.WITH_WORKDIR`` is ``True`` git-new-workdir will be installed.
    
    Lastly, it will append render the template ``activate-profile.tpl`` and
    append it to ``bin/activate``. The script will setup common needed
    environment variables that e.g. invenio-devscripts depend on.
    
    If an existing environment already exists, the user will be asked for
    confirmation to remove the directory (using sudo, due to the directory
    ``var/tmp/ooffice-tmp-files`` which is created using sudo).
    """
    # Checks
    if 'CFG_INVENIO_PREFIX' not in env:
        abort(red("CFG_INVENIO_PREFIX is not specified in env.") % env)

    puts(cyan(">>> Creating virtualenv in %(CFG_INVENIO_PREFIX)s..." % env))

    # Remove previous installation
    if os.path.exists(env.CFG_INVENIO_PREFIX):
        res = confirm("Remove installation in %(CFG_INVENIO_PREFIX)s ?" % env)
        if not res:
            abort(red("Cannot continue") % env)
        else:
            local("sudo rm -Rf %(CFG_INVENIO_PREFIX)s" % env)

    # Create virtual environment
    dirname = os.path.dirname(env.CFG_INVENIO_PREFIX)
    basename = os.path.basename(env.CFG_INVENIO_PREFIX)

    local("mkdir -p %s" % dirname)
    local("cd %s && virtualenv -p %s %s" % (dirname, env.PYTHON, basename))

    # Create needed symboic links
    pyver = python_version()
    local("mkdir -p %(CFG_INVENIO_PREFIX)s/lib/python/invenio" % env)
    local(("mkdir -p %(CFG_INVENIO_PREFIX)s/lib/python" + pyver + "/site-packages") % env)
    local(("ln -s %(CFG_INVENIO_PREFIX)s/lib/python/invenio %(CFG_INVENIO_PREFIX)s/lib/python" + pyver + "/site-packages/invenio") % env)

    # Write extras into the activate script
    write_template(os.path.join(env.CFG_INVENIO_PREFIX, 'bin/activate'), env, tpl_file='activate-profile.tpl', append=True, mark="ACTIVATE_PROFILE")

    # Install devscripts
    if env.WITH_DEVSCRIPTS:
        puts(">>> Installing invenio-devscripts...")
        local("cd %(CFG_INVENIO_PREFIX)s && git clone https://github.com/tiborsimko/invenio-devscripts.git" % env)
        local("cd %(CFG_INVENIO_PREFIX)s && mv invenio-devscripts/* bin/" % env)

    if env.WITH_WORKDIR:
        puts(">>> Installing git-new-workdir...")
        local('wget -O %(CFG_INVENIO_PREFIX)s/bin/git-new-workdir "http://repo.or.cz/w/git.git/blob_plain/HEAD:/contrib/workdir/git-new-workdir"' % env)
        local("chmod +x %(CFG_INVENIO_PREFIX)s/bin/git-new-workdir" % env)

    # OpenOffice temporary directory
    local("mkdir -p %(CFG_INVENIO_PREFIX)s/var/tmp/ooffice-tmp-files" % env)
    local("sudo chown -R nobody %(CFG_INVENIO_PREFIX)s/var/tmp/ooffice-tmp-files" % env)
    local("sudo chmod -R 755 %(CFG_INVENIO_PREFIX)s/var/tmp/ooffice-tmp-files" % env)


@task
def venv_drop():
    """ Drop virtualenv environment """
    # Checks
    if 'CFG_INVENIO_PREFIX' not in env:
        abort(red("CFG_INVENIO_PREFIX is not specified in env.") % env)

    puts(cyan(">>> Dropping virtualenv in %(CFG_INVENIO_PREFIX)s..." % env))

    # Remove previous installation
    if os.path.exists(env.CFG_INVENIO_PREFIX):
        res = confirm("Remove installation in %(CFG_INVENIO_PREFIX)s ?" % env)
        if not res:
            abort(red("Cannot continue") % env)
        else:
            local("sudo rm -Rf %(CFG_INVENIO_PREFIX)s" % env)
    else:
        puts(">>> Nothing to remove - %(CFG_INVENIO_PREFIX)s does not exists..." % env)




# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Fabric tasks for bootstrapping, installing, deploying and running Atlantis.

Examples
--------
Setup virtual environment and database with Python 2.4 and Invenio v1.1.0:
  fab loc:py=24,ref=origin/v1.1.0 bootstrap
  fab loc:py=24,ref=origin/v1.1.0 invenio_create_demosite

Dump, load and drop (database and virtual environment):
  fab loc:py=27,ref=master dump
  fab loc:py=27,ref=master load
  fab loc:py=27,ref=master drop

Requirements:
 * pythonbrew must be installed to specify a specific Python version.
"""

from fabric.api import task, abort
from fabric.colors import red
from inveniofab.api import *
import os

@task
def loc(activate=True, py=None, ref=None, **kwargs):
    """
    Local environment (example: loc:py=24,ref=maint-1.1)
    """
    if 'name' not in kwargs:
        kwargs['name'] = env_make_name('atlantis', py or '', ref or '')

    env = env_create('loc', activate=activate, python=py, **kwargs)

    return env_override(env, 'invenio', ref)



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.


"""
Fabric tasks for bootstrapping, installing, deploying and running INSPIRE.

Examples
--------
Bootstrap INSPIRE with Python 2.6
  fab loc:py=26 bootstrap

Configure OpenAIRE repository to use with environment
  fab loc:py=26,ref=master repo_configure:inspire
"""

from fabric.api import task, puts, local
from fabric.colors import cyan
from inveniofab.api import *
from inveniofab.utils import template_hook_factory
import os


@task
def loc(activate=True, py=None, ref=None, invenio='master', **kwargs):
    """
    Local environment (example: loc:py=24,ref=master)
    """
    env = env_create('loc', name=env_make_name('inspireprod', py or '', ref or ''),
                     activate=activate, python=py, **kwargs)

    # Do not use git-new-workdir
    env.WITH_WORKDIR = False
    env.CFG_SRCWORKDIR = env.CFG_SRCDIR

    # FIXME: Should probably be generalized via hooks in the bootstrap task instead.
    # Adds specific inspire steps to, e.g bootstrap task
    env.CFG_INSPIRE_SITE = 1

    # Setup Invenio/INSPIRE source directories (works just as short-cuts)
    # Their path must match CFG_SRCDIR/<reponame> where <reponame> is defined
    # in CFG_INVENIO_REPOS.
    env.CFG_INVENIO_SRCDIR = os.path.join(env.CFG_SRCDIR, 'invenio-inspire-ops')
    env.CFG_INSPIRE_SRCDIR = os.path.join(env.CFG_SRCDIR, 'inspire')

    env.CFG_INVENIO_REPOS = [
        ('invenio-inspire-ops', {
            'repository': 'ssh://%(user)s@lxplus.cern.ch/afs/cern.ch/project/inspire/repo/invenio-inspire-ops.git' % env,
            'ref': 'prod',
            'bootstrap_targets': ['all', 'install', 'install-mathjax-plugin', 'install-ckeditor-plugin', 'install-pdfa-helper-files', 'install-jquery-plugins', ],
            'deploy_targets': ['all', 'install', ],
            'requirements': ['requirements.txt', 'requirements-extra.txt']
        }),
        ('inspire', {
            'repository': 'ssh://%(user)s@lxplus.cern.ch/afs/cern.ch/user/s/simko/public/repo/inspire.git' % env,
            'ref': 'master',
            'bootstrap_targets': ['install', ],
            'deploy_targets': ['install', ],
            'configure_hook': template_hook_factory('config-local.mk', '%(topsrcdir)s/config-local.mk'),
            'prepare_hook': None
        }),
    ]

    env.CFG_DATABASE_HOST = 'localhost'
    env.CFG_DATABASE_PORT = 3306
    env.CFG_INVENIO_ADMIN = 'javier.martin.montull@cern.ch'

    # If no database or username is specified a new database with the same
    # name as the virtualenv will be created. Also a user with the env username
    # will be created
    #env.CFG_DATABASE_NAME = 'invenio'
    #env.CFG_DATABASE_USER = 'invenio'

    # Set default Invenio branch
    env = env_override(env, 'invenio', invenio, override={})
    return env

# =======================
# INSPIRE specific tasks
# =======================
@task
def inspire_dbchanges():
    """ Perform INSPIRE db changes """
    from fabric.api import env
    puts(cyan(">>> Updating tables from INSPIRE..." % env))
    ctx = {
        'topsrcdir': repo_check('inspire', check_path=True, workdir=True),
    }
    ctx.update(env)
    local("cd %(topsrcdir)s; make install-dbchanges" % ctx)

    query_output = local('echo "select max(id) from schTASK;" | dbexec', capture=True)
    task_id = query_output.split('\n')[-1]
    puts(cyan(">>> Running webcoll task %s..." % task_id))

    ctx.update({'webcoll_task_id': task_id})
    local("%(CFG_INVENIO_PREFIX)s/bin/webcoll %(webcoll_task_id)s" %
          ctx)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Fabric tasks for defining environments.

Fabric environment dictionary
---------------------------
An integral part of Fabric is what is know as "environments": a Python dictionary
subclass which is used as a combination settings registry and shared inter-task
data namespace. Most of Fabric's and Invenio Fabrics behaviour is modifiable 
through env variables.

The environment is accessible through ``fabric.api.env``, and allows e.g. a task
access to the current user etc (``user`` is called an env variable)::

  from fabric.api import env
  @task
  def sometask():
      print env.user
      print env.host

For more information on Fabric environment dictionary please see 
http://fabric.readthedocs.org/en/latest/usage/env.html

Invenio Fabric environments
---------------------------
The tasks in this module helps you define the env variables which all other
Invenio Fabric tasks depends on. Secondly, it provide means for loading 
env variables, but without putting them in ``fabric.api.env``. This is useful
for tasks like ``fab int mysql_copy:prod`` which would copy the database from
production to integration, and thus need access to host, user, password etc from
both the integration and production environment at the same time. 

Usage
^^^^^
Defining a new environment is simple::

  from fabric.api import task
  from inveniofab.api import *

  @task
  def prod(activate=True):
      # Step 1: Create env with default values
      env = env_create('prod', activate=activate)

      # Step 2: Modify some values
      env.CFG_XYZ = ...

      # Step 3: Return env
      return env

Things to note:

 * The task must take a keyword argument ``activate`` which defaults to ``True``.
   This is used to control if the environment is activated or not (i.e. copied
   to ``fabric.api.env``.
 * First parameter to :meth:`~env_create`, must match the task name (``prod`` in
   this case). Otherwise the environment cannot be loaded without activating it,
   and template overriding will not work properly (see later in this chapter).
 * You must return the environment variable at the end of the task.

Running tasks
^^^^^^^^^^^^^
As mentioned above, the Invenio Fabric tasks depends on an environment being
defined. The pattern to run Invenio Fabric task is::

  fab <env> <task1> <task2>

In the example above, you could for instance run::

  fab prod bootstrap

to bootstrap the production environment. I.e. you always call an environment 
task as the first task.

Also note, that some task takes parameters::

  fab <env> <task1>:<arg1>,<arg2>,<kw1>=<val>,<kw2>=<val> <task2> ...
  fab prod bootstrap:quite=1

Roles
^^^^^
By default the following roles are defined:

 * ``web`` - frontend webservers
 * ``lb`` - load balancers
 * ``db-master`` - master databases
 * ``db-slave`` - slave databases
 * ``workers`` - worker nodes

Some task will use these roles to decide which machine to execute a given 
command on. For more on Fabric roles please see 
http://fabric.readthedocs.org/en/1.4.3/usage/execution.html#roles

You may override these roles in your environment task::

  @task
  def prod(activate=True):
      env = env_create('prod', activate=activate)
      env.roledefs['web'] = [...]
      return env 

Creating tasks
--------------
When creating task you can access the variables simply by loading the Fabric
env dictionary from ``fabric.api.env``. All Invenio Fabric env variables are 
upper-case (for now, please look in the source code of :meth:`~env_defaults` to 
see, which variables are defined).

Example::

  from fabric.api import env

  @task
  def sometask():
      print env.CFG_INVENIO_PREFIX
      local("%(CFG_INVENIO_PREFIX)s/bin/python" % env)


Template loading
^^^^^^^^^^^^^^^^
The Invenio Fabric environment by default also configures a Jinja2 environment
which can be used to load and render templates.

Example::

  from fabric.api import env

  @task
  def sometask():
      tpl = env.jinja.get_template("etc/invenio-local.conf")
      output = tpl.render(env)
      ...

Above code will look for the template first in ``<env>/etc/invenio-local.conf``,
then ``common/etc/invenio-local.conf``. So assuming your directory structure
looks like below::

  fabfile.py
  common/etc/invenio-local.conf
  int/etc/invenio-local.conf

Running ``fab int sometask`` would use the template 
``int/etc/invenio-local.conf``, while ``fab prod sometask`` and 
``fab loc sometask`` would use the template ``common/etc/invenio-local.conf``.

This allows you to define common templates for configuration files, and 
selectively override them for certain environments if needed.


Relation to invenio-local.conf
------------------------------
Many variables map directly to their Invenio counterpart - e.g.
``CFG_DATABASE_NAME`` can be defined in invenio-local.conf as well as in
Invenio Fabric ``env.CFG_DATABASE_NAME``.

Usually when deploying or working with a specific Invenio project, some 
configuration variables depend only on the project (e.g. 
``CFG_WEBSTYLE_TEMPLATE_SKIN``), while others depend
on the deployment environment (e.g. ``CFG_DATABASE_HOST``).

As a general rule, project-wide configuration should go in a common 
``invenio-local.conf`` template, while deployment dependent configuration should
be defined in a Fabric environment task. The ``invenio-local.conf`` can then be
render with the Fabric env variables.

Tasks
-----
"""

from fabric import state
from fabric.api import execute, abort, task
from fabric.api import task, abort
from inveniofab.utils import pythonbrew_versions
from jinja2 import Environment, FileSystemLoader
import copy
import os

GLOBAL_REFS_OVERRIDE = {
    'invenio' : {
        'next' : {
            'bootstrap_targets': ['all', 'install', 'install-mathjax-plugin',
                'install-ckeditor-plugin', 'install-pdfa-helper-files',
                'install-jquery-plugins', 'install-jquery-tokeninput',
                'install-bootstrap' ],
            'deploy_targets': ['all', 'check-upgrade', 'install', ],
            'requirements' : ['%(CFG_INVENIO_SRCDIR)s/requirements.txt',
                '%(CFG_INVENIO_SRCDIR)s/requirements-extras.txt',
                '%(CFG_INVENIO_SRCDIR)s/requirements-flask.txt',
                '%(CFG_INVENIO_SRCDIR)s/requirements-flask-ext.txt',],
        },
        'origin/v0.99.0' : {
            'bootstrap_targets' : ['all', 'install'],
            'deploy_targets' : ['all', 'install'],
        },
        'origin/v0.99.5' : {
            'bootstrap_targets' : ['all', 'install'],
            'deploy_targets' : ['all', 'install'],
        },
    },
}
"""
Overrides installation procedure for a specific branch.

Certain branches might have special installation requirements which does not
match the default. This dictionary allows you to override the default values.
"""


def env_create(envname, defaults_func=None, activate=True, **kwargs):
    """
    Setup a new environment (e.g. integration, production, local).
    
    See module documentation above for detailed explanation of enviroments.
    
    :param envname: str, Name of environment (must be same as task envname).
    :param defaults_func: callable taking a dictionary as argument and returns
        the same dictionary. Used to setup defaults in the environment. By 
        default the :meth:`~env_defaults` is used. Take great care if overriding
        this, since many tasks expects specific variables to be defined.
    :param activate: True to activate the environment, or False to just
        load it (i.e. should config be put in ``fabric.api.env`` or not).
    """
    from fabric.api import env

    if activate:
        some_env = env
        some_env.env_active = True
    else:
        tmp = env.jinja
        env.jinja = None

        some_env = copy.deepcopy(env)
        some_env.env_active = False

        env.jinja = tmp

    some_env.env_name = envname

    # Allow user to setup defaults
    if defaults_func:
        some_env = defaults_func(some_env, **kwargs)
    else:
        some_env = env_defaults(some_env, **kwargs)

    return some_env


def env_get(name):
    """
    Get environment by name (does not activate the environment).

    An environment is defined in a local task. This task will
    look for a task with the name <name> and execute. Hence
    the task defining the environment.
    
    Example::
    
        from fabric.api import env

        @task
        def sometask():
            another_env = env_get('prod')
            print env.CFG_SITE_URL
            print another_env.CFG_SITE_URL

    Running ``fab int sometask`` would print first the integration 
    ``CFG_SITE_URL``, then the production ``CFG_SITE_URL``.
    """
    from fabric.api import env

    # Check if active env is requested
    if env.get('env_active', False) and env.get('env_name', None) == name:
        return env

    if 'loaded_envs' not in env:
        env.loaded_envs = {}

    # Check if env is already loaded, if not load it.
    if name not in env.loaded_envs:
        res = execute(name, activate=False)
        env.loaded_envs[name] = res['<local-only>']

    return env.loaded_envs[name]


def env_defaults(env, name='invenio', prefix=None, python=None, **kwargs):
    """
    Setup defaults in environment.

    The method will by default try to guess
    
    :param name:
    :param prefix:
    :param python:

    .. note:
    
       This can be overridden by the user, though it is not recommended.
    """
    # Initialise template loader
    env.jinja = Environment(loader=FileSystemLoader([env.env_name, 'common']))
    prefix = prefix or os.path.join(os.getenv('WORKON_HOME', '/opt'), name)

    env.roledefs = {
        'web': [],
        'lb': [],
        'db-master': [],
        'db-slave': [],
        'workers': [],
    }

    if python is None:
        pythonbin = 'python'
    else:
        python_versions = pythonbrew_versions()
        if python in python_versions:
            pythonbin = python_versions[python]
        else:
            python_versions = python_versions.keys()
            python_versions.sort()
            abort("Unknown Python version %s. Available versions are %s" % (
                  python, ", ".join(python_versions)))

    env.update({
        'WITH_VIRTUALENV': True,
        'WITH_DEVSERVER': True,
        'WITH_WORKDIR': True,
        'WITH_DEVSCRIPTS': True,
        'PYTHON': pythonbin,
        'ACTIVATE': '. %(CFG_INVENIO_PREFIX)s/bin/activate',
        'CFG_INVENIO_REPOS': [],
        'CFG_INVENIO_CONF': 'etc/invenio-local.conf',
        'CFG_INVENIO_PREFIX': prefix,
        'CFG_INVENIO_SRCDIR' : os.path.join(prefix, 'src/invenio'),
        'CFG_SRCDIR' : os.environ.get('CFG_SRCDIR', os.path.join(prefix, 'src')),
        # Only use CFG_SRCWORKDIR if WITH_WORKDIR is True
        'CFG_SRCWORKDIR' : os.path.join(prefix, 'src'),
        'CFG_INVENIO_HOSTNAME': "localhost",
        'CFG_INVENIO_DOMAINNAME': "",
        'CFG_INVENIO_PORT_HTTP': "4000",
        'CFG_INVENIO_PORT_HTTPS': "4000",
        'CFG_INVENIO_USER': env.user,
        'CFG_INVENIO_APACHECTL': '/etc/init.d/httpd',
        'CFG_INVENIO_ADMIN': 'nobody@localhost',

        'CFG_DATABASE_DUMPDIR': prefix,
        'CFG_DATABASE_HOST': 'localhost',
        'CFG_DATABASE_PORT': 3306,
        'CFG_DATABASE_NAME': name,
        'CFG_DATABASE_USER': name,
        'CFG_DATABASE_PASS': 'my123p$ss',
        'CFG_DATABASE_DROP_ALLOWED': True,

        # FIXME: Set to localhost or leave empty?
        'CFG_MISCUTIL_SMTP_HOST': '127.0.0.1',
        'CFG_MISCUTIL_SMTP_PORT': '1025',

        # FIXME: Should be removed, so Inspire stuff is not integrated in main
        # source file.
        'CFG_INSPIRE_SITE' : False,
    })

    env.CFG_INVENIO_REPOS = [
        ('invenio', {
            'repository' : 'http://invenio-software.org/repo/invenio/',
            'ref': 'master',
            'bootstrap_targets': ['all', 'install', 'install-mathjax-plugin', 'install-ckeditor-plugin', 'install-pdfa-helper-files', 'install-jquery-plugins', ],
            'deploy_targets': ['all', 'check-upgrade', 'install', ],
            'requirements': ['requirements.txt', 'requirements-extra.txt', ],
            #'configure_hook': None, # Can be specified if you want to override default behaviour (./configure --with-prefix=.. --with-python=..)
            #'prepare_hook': None, # Can be specified if you want to override default behaviour  (aclocal && autoconf && automake)
        }),
    ]

    return env


def env_override(env, this_repo, this_ref, override={}, global_overrides=None ):
    """ Override default values for repository """
    if global_overrides:
        try:
            this_repo_dict = global_overrides[this_repo][this_ref]
        except KeyError:
            return env
    elif override:
        this_repo_dict = override
    else:
        try:
            this_repo_dict = GLOBAL_REFS_OVERRIDE[this_repo][this_ref]
        except KeyError:
            return env

    def _mapper(x):
        repo, repo_dict = x

        if repo == this_repo:
            repo_dict.update(this_repo_dict)
            if this_ref:
                repo_dict['ref'] = this_ref
        return (repo, repo_dict)

    env.CFG_INVENIO_REPOS = map(_mapper, env.CFG_INVENIO_REPOS)

    return env


def env_make_name(prefix, python, ref):
    """ Generate a MySQL friendly environment name. """
    ref = ref.split("/")
    ref = ref[-1].replace("-","_")
    python = python.replace(".","")
    
    prefix = prefix.replace("_","").replace("-","")

    name = "%s%s%s" % (prefix, python, ref)
    if len(name) > 16:
        name = name.replace("_", "", len(name) - 16)
        return name[:16]

    return name
